%Solves a belonging problem for a given square grid of size N, with A
%angles
N = 15;
A = 6;

ANGCONV = pi/180;

%a shape is defined by two lines
ang = 90/A*ANGCONV;
%since tan90 is inf, we find the largest practical value
angarr = [ang*(0:1:A-1) atan(N)];
SHAPES = [tan(angarr(1:A))' tan(angarr(2:end))'];

%a box is defined by a structure containing fields for its assigned shape,
%a probability and 25 values of internal points
points = zeros(25,2);
pts = linspace(0,1,5);
for i = 0:1:4
    %x values are constant
points((i*5)+(1:5),1) = pts(i+1)*ones(1,5);
points((i*5)+(1:5),2) = pts;
end


%Draw boxes for a visual check
M = colormap('hsv');
M = M(floor(linspace(1,length(M),A+1)),:);
M(1,:) = [ 0 0 0];
close all;
figure;
hold on;
for i = 1:N+1
    plot(0:N,(i-1)*ones(1,N+1),'k');
    plot((i-1)*ones(1,N+1),0:N,'k');
end
AXIS([0 N 0 N])
for i = 1:A
    plot(0:N, tan(angarr(i))*(0:N),'b');
end


BOXES = cell(N,N);
for i = 1:N
   for j = 1:N
      %create the box in position (i,j) 
      xoffset = (i-1);
      yoffset = (j-1);
      bpoints = [(points(:,1)+xoffset) (points(:,2)+yoffset)];
      
      %check for each point, which shape it belongs to
      chk = zeros(25,1);
      for k = 1:25
         yt = bpoints(k,1)*SHAPES;
         t = (bpoints(k,2) >= yt(:,1)) & (bpoints(k,2) <= yt(:,2));
         if (sum(t) > 1 || sum(t) == 0)
            %belongs to shape 0
            chk(k) = 0;
         else
             chk(k) = find(t==1);
         end
      end
      
      %add no go zones:
      dist = sqrt(bpoints(:,1).^2 + bpoints(:,2).^2);
      distchk = (dist < (N/6));
      chk(distchk) = 0;
      %distchk = (dist < (N/4)) | (dist > (110*N/100));
      %chk(distchk) = 0;
      
      %determine if there exists a majority value
      candidates = unique(chk);
      candidatecnt = zeros(1,length(candidates));
      for k = 1:length(candidates)
            candidatecnt(k) = sum(chk==candidates(k));
      end
      [p ind] = (max(candidatecnt));
      p = p/25;
      if p < 0.4
          s = 1
      else
      s = candidates(ind)+1;
      end
      
      box = struct('point', bpoints, 'shape', s, 'prob', p)
      BOXES(i,j) = {box};
   end
end

%clean up the boxes a bit: no go zones
for i = 6:N
    BOXES{1,i}.shape = A+1;
end


%draw boxes w assignments
for i = 1:N
    for j = 1:N
      %mark box by a colored x
      p = BOXES{i,j}.prob;
      s = BOXES{i,j}.shape;
      plot(i-0.5,j-0.5,'o','MarkerFaceColor',M(s,:),'MarkerEdgeColor',M(s,:));
    end
end



